Attribute VB_Name = "modCompress06"
Option Explicit

Public Function Compress06( _
            sExpression As String, _
            sCompress As String, _
            Optional Compare As VbCompareMethod = vbBinaryCompare) As String

' by Tom Winters, tom@interplanetary.freeserve.co.uk, 20011104

    Dim sExp$, sFind$, lLenCompress&, lLenExpression&
    Dim lChrPosition&

    lLenExpression = Len(sExpression)
    If lLenExpression = 0 Then Exit Function
    lLenCompress = Len(sCompress)

    '//---------------------------------------------------------------------------
    '// Compress single-character search string in short expression
    '//---------------------------------------------------------------------------

    If lLenCompress <> 0 Then

        '// If searchstring is one character in length
        If lLenCompress = 1 Then

            '// If expression is reasonably short string
            If lLenExpression < 10 Then
            
            '// Double size of compress string
            sFind = sCompress + sCompress
            
                '// Loop till expression is compressed
                Compress06 = sExpression
                Do
                    '// This method is fast with short strings
                    lChrPosition = InStr(1, Compress06, sFind, Compare)
                    If lChrPosition = 0 Then Exit Function
                    sExp = Left$(Compress06, lChrPosition)
                    Compress06 = Right$(Compress06, Len(Compress06) - _
                                                                Len(sExp) - lLenCompress)
                    Compress06 = sExp + Compress06
                Loop
            Else
                '// ..we must have a long expression or multiple search string

                '//---------------------------------------------------------------------------
                '// Test for repetative search string
                '//---------------------------------------------------------------------------

                '// Ideally we'd check the the entire string for segment matches,
                '// but if we do that we'll be here for ever
                '// So, we'll use a reasonable compromise..
                
                '// Test first 12 characters of expression for a 66%
                '// repetative compress string

                Dim sNewSearchString$

                sExp = Left$(sExpression, 12)
                sNewSearchString = String$(8, sCompress)
                lChrPosition = InStr(1, sExp, sNewSearchString, Compare)
                
                '// If we have a large repetative string
                If lChrPosition > 0 Then
                
                '//---------------------------------------------------------------------------
                '// Compress repetative single-character search string in long expression
                '//---------------------------------------------------------------------------
                
                    Dim lLenNewSearchString&, lLenFind2&, lStringSizeCounter&

                    lLenFind2 = lLenCompress + lLenCompress
                    lStringSizeCounter = (lLenExpression - lLenFind2)

                    '// Make new search string divisible by 2
                    lStringSizeCounter = lStringSizeCounter + (lStringSizeCounter And 1)
                    
                    '// Build new search string
                    sNewSearchString = String$(lStringSizeCounter, sCompress)
                    lLenNewSearchString = Len(sNewSearchString)
                    lStringSizeCounter = 0
                    Compress06 = sExpression
                    sFind = sCompress + sCompress

                    '// If we're looking for a large string, then faster to start
                    '// searching with a large string than with a small string
                    '// So, start with a large search-string, progressively
                    '// reducing length till match is found

                    '// This method is fast with long repetative strings
                    Do
                        Do
                            lChrPosition = InStr(1, Compress06, sNewSearchString, Compare)
                            If lChrPosition = 0 Then Exit Do
                            sExp = Left$(Compress06, lChrPosition)
                            Compress06 = Right$(Compress06, Len(Compress06) _
                                          - Len(sExp) - lLenNewSearchString + lLenCompress)
                            Compress06 = sExp + Compress06
                            
                            '// Reset quantity removed from search string
                            lStringSizeCounter = 0
                        Loop
                        lChrPosition = InStr(1, Compress06, sFind, Compare)
                        If lChrPosition = 0 Then Exit Function
                        
                        '// Increase size of search string counter
                        lStringSizeCounter = lStringSizeCounter + lLenCompress
                        
                        '// Make search string smaller and try again
                        sNewSearchString = Right$(sNewSearchString, Len(Compress06) _
                                                                      - lStringSizeCounter)
                        lLenNewSearchString = Len(sNewSearchString)
                    Loop
                End If
            End If
        End If

        '//---------------------------------------------------------------------------
        '// Check for Unicode
        '//---------------------------------------------------------------------------

        '// We can save a lot of work by not passing Unicode strings to our byte arrays
        '// so we test for Unicode first and if true, use another Instring method

        Dim lCharacter&, lAsciiValue&

        For lCharacter = 1 To lLenCompress

            lAsciiValue = Asc(Mid$(sCompress, lCharacter, 1))
            If lAsciiValue > 127 Then
                '// If we have a Unicode multiple search character

                Dim bGo As Boolean, lPosition&
                sExp = sExpression
                
                '// This method is fast with more complex expressions
                Do While Len(sExp) > 0
                    bGo = False
                    lPosition = InStr(1, sExp, sCompress, Compare)
                    If Mid$(sExp, lPosition + lLenCompress, lLenCompress) = sCompress Then
                        If lPosition = 1 Then
                            bGo = True
                        End If
                    End If
                    If bGo Then
                        sExp = Right$(sExp, Len(sExp) - lLenCompress)
                    Else
                        Compress06 = Compress06 + Left$(sExp, 1)
                        sExp = Right$(sExp, Len(sExp) - 1)
                    End If
                Loop
                Exit Function
            End If
        Next

        '//---------------------------------------------------------------------------
        '// Compress using byte arrays
        '//---------------------------------------------------------------------------

        Dim bMatch As Boolean, bMatchResult1 As Boolean, bMatchResult2 As Boolean
        Dim lLenExpressionArray&, lLenCompressArray&, lbytePosition&, lNewCounter&
        Dim byExpressionArray() As Byte, byNewArray() As Byte, byCompressArray() As Byte
        Dim lNearEndofExpression&, lExpCounter&, lLenCompressArrayplus1&
        
        '// Set case according to status of comparison
        If Compare = vbTextCompare Then
            sExpression = LCase$(sExpression)
            sCompress = LCase$(sCompress)
        End If

        '// Convert string to byte array
        byExpressionArray = sExpression
        byCompressArray = sCompress

        '// Get byte array length from previously found string length
        '// (tad faster than Ubound)
        lLenExpressionArray = lLenExpression + lLenExpression - 1
        lLenCompressArray = lLenCompress + lLenCompress - 1
        
        '// Resize new array
        ReDim byNewArray(lLenExpressionArray)
        lNewCounter = 0

        '// Resolve first character match to init loop
        bMatch = Left$(sExpression, 1) = sCompress

        '//---------------------------------------------------------------------------
        '// Compress single-character search string using byte array
        '//---------------------------------------------------------------------------

        '// Equate character match in boolean terms, applying logic "And"
        If Not bMatch And (lLenCompressArray = 1) Then
            For lbytePosition = 1 To lLenExpressionArray

                '// For/Next loops work faster advancing from 1
                '// But we want lbytePosition to start at byte zero so;
                lbytePosition = lbytePosition - 1
                If byExpressionArray(lbytePosition) <> byCompressArray(0) Then
                
                    '// Set this byte of byNewArray() equal to byExpressionArray()
                    byNewArray(lNewCounter) = byExpressionArray(lbytePosition)
                    lNewCounter = lNewCounter + 2
                Else
                
                    '// Set this byte of byNewArray() equal to byCompressArray()
                    If byExpressionArray(lbytePosition - 2) <> byCompressArray(0) Then
                        byNewArray(lNewCounter) = byCompressArray(0)
                        lNewCounter = lNewCounter + 2
                    End If
                End If

                '// As we want to advance 2 Hex places on each loop cycle
                '// (the range of 1 byte being 0-255 (&H0 to &HFF, 2^8 or 8-bits))
                '// we add 2 to lbytePosition, (a touch faster than using "Step 2")
                lbytePosition = lbytePosition + 2
            Next
        Else

            '//---------------------------------------------------------------------------
            '// Compress multi-character search string using byte array
            '//---------------------------------------------------------------------------

            '// For a 3 character compress string (could be any length here)
            '// if the expression contains "xbz" and the findstring is "abc"
            '// bMatchResult1 of ("x" And "a") = False
            '// bMatchResult2 of ("b" And "b") = True
            '// We "And" these two results so that (in this example);
            '// (bMatchResult1 And bMatchResult2) = (False And True) = False
            '// Pass this result to bMatchResult1 and pass a new test to bMatchResult2
            '// bMatchResult2 result: ("z" And "c") = False
            '// -> (bMatchResult1 And bMatchResult2) = (False And False) = False!
            
            '// We only get a result of True, if all characters in the stream match

            lNewCounter = 0
            lLenCompressArrayplus1 = lLenCompressArray + 1
            lNearEndofExpression = lLenExpressionArray - (lLenCompressArray - 1)
            bMatchResult1 = True

            '// lbytePosition is at start of expression, so we init subsequent loop by
            '// obtaining a match result for bFindArray() in bExpressionArray()
            For lbytePosition = 1 To lLenCompressArrayplus1
                lbytePosition = lbytePosition - 1

                '// Equate character match in boolean terms
                bMatchResult2 = byExpressionArray(lbytePosition) = byCompressArray(lbytePosition)

                '// Colate results to date and save
                bMatchResult1 = bMatchResult1 And bMatchResult2

                '// Provided no match was found in stream
                If Not bMatchResult1 Then

                    '// Reset byNewArray() counter to zero and exit loop
                    lNewCounter = 0
                    Exit For
                End If

                '// ..or advance both array counters and check next character
                byNewArray(lNewCounter) = byCompressArray(lbytePosition)
                lNewCounter = lNewCounter + 2
                lbytePosition = lbytePosition + 2
            Next

            '// With this result for first term done, we loop to build bNewArray()
            For lExpCounter = 1 To lLenExpressionArray
                lExpCounter = lExpCounter - 1

                '// Reset current result to False
                bMatch = False

                '// If we're not at end of the expression
                If lExpCounter < lNearEndofExpression Then
                    bMatch = True

                    '// For each element in byCompressArray(), check for match with expression
                    For lbytePosition = 1 To lLenCompressArray
                        lbytePosition = lbytePosition - 1
                        bMatchResult2 = byExpressionArray(lExpCounter + lbytePosition) _
                                                            = byCompressArray(lbytePosition)
                        bMatch = bMatch And bMatchResult2
                        lbytePosition = lbytePosition + 2
                    Next
                End If

                '// If no match found, set just one byte in byNewArray()
                '// to current byExpressionArray() byte value..
                If Not bMatch Then
                    byNewArray(lNewCounter) = byExpressionArray(lExpCounter)

                    '// ..and advance both counters to next byte
                    lNewCounter = lNewCounter + 2
                    lExpCounter = lExpCounter + 2

                '// Provided the previous pass through loop did not achieve a match
                ElseIf Not bMatchResult1 Then
                    
                    '// Work through find string stream [ bFindArray() ]
                    For lbytePosition = 1 To lLenCompressArray
                        lbytePosition = lbytePosition - 1
                        
                        '// Set current byte in byNewArray() = byCompressArray()
                        '// and advance all counters
                        byNewArray(lNewCounter) = byCompressArray(lbytePosition)
                        lNewCounter = lNewCounter + 2
                        lExpCounter = lExpCounter + 2
                        lbytePosition = lbytePosition + 2
                    Next
                Else

                    '// If no match was found
                    '// just advance lExpressionCounter beyond length of search string
                    lExpCounter = lExpCounter + lLenCompressArrayplus1
                End If
                
                '// ..accumulating results thus far
                bMatchResult1 = bMatch
                
            '// ..and try again
            Next
        End If

        '//---------------------------------------------------------------------------
        '// Convert byNewArray() to String
        '//---------------------------------------------------------------------------

        Compress06 = byNewArray

        '// ..and resize string to correct length
        '// (slightly faster than using ReDim Preserve on byte array)
        Compress06 = Left$(Compress06, lNewCounter * 0.5)
        Exit Function
    Else

        '//---------------------------------------------------------------------------
        '// Handle Error
        '//---------------------------------------------------------------------------
        
        '// If compress string was of zero length set function equal to expression
        Compress06 = sExpression

    End If

End Function


